/* exported init buildPrefsWidget */
const { Gdk, Gio, GObject, Gtk } = imports.gi;

const ExtensionUtils = imports.misc.extensionUtils;
const Gettext = imports.gettext.domain('gnome-shell-extensions');
const _ = Gettext.gettext;

let settings = null;

var PreferencesGroup = GObject.registerClass(
class PreferencesGroup extends Gtk.ListBox {
    _init(params) {
        super._init({
            ...params,
            selection_mode: Gtk.SelectionMode.NONE,
        });

        const styleContext = this.get_style_context();
        styleContext.add_class('frame');
        styleContext.add_class('rich-list');

        this.set_header_func(this._updateHeader.bind(this));
        this.connect('row-activated',
            (box, row) => row.activate());
    }

    _updateHeader(row, before) {
        if (!before || row.get_header())
            return;
        row.set_header(new Gtk.Separator());
    }
});


var ActionRow = GObject.registerClass({
    Properties: {
        'activatable-widget': GObject.ParamSpec.object(
            'activatable-widget', 'activatable-widget', 'activatable-widget',
            GObject.ParamFlags.READWRITE,
            Gtk.Widget),
        'title': GObject.ParamSpec.string(
            'title', 'title', 'title',
            GObject.ParamFlags.READWRITE,
            null),
    },
}, class ActionRow extends Gtk.ListBoxRow {
    _init(params) {
        super._init(params);

        const box = new Gtk.Box({
            valign: Gtk.Align.CENTER,
            spacing: 12,
            margin_top: 8,
            margin_bottom: 8,
            margin_start: 12,
            margin_end: 12,
        });
        this.add(box);

        const provider = new Gtk.CssProvider();
        provider.load_from_data('* { min-height: 32px; }');
        box.get_style_context().add_provider(provider,
            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION);

        this._prefixes = new Gtk.Box({
            spacing: 12,
            no_show_all: true,
            visible: false,
        });
        box.add(this._prefixes);

        this._title = new Gtk.Label({
            hexpand: true,
            xalign: 0,
        });
        this._title.get_style_context().add_class('title');
        box.add(this._title);

        this._suffixes = new Gtk.Box({
            spacing: 12,
            no_show_all: true,
            visible: false,
        });
        box.add(this._suffixes);

        this.bind_property('title',
            this._title, 'label',
            GObject.BindingFlags.SYNC_CREATE);
    }

    activate() {
        if (this.activatable_widget)
            this.activatable_widget.mnemonic_activate(false);
    }

    add_prefix(child) {
        child.show();
        this._prefixes.add(child);
        this._prefixes.show();
    }

    add_suffix(child) {
        child.show();
        this._suffixes.add(child);
        this._suffixes.show();
    }
});

var ComboRow = GObject.registerClass({
    Properties: {
        'selected-item': GObject.ParamSpec.object(
            'selected-item', 'selected-item', 'selected-item',
            GObject.ParamFlags.READABLE,
            GObject.Object),
    },
}, class ComboRow extends ActionRow {
    _init(params) {
        super._init({
            ...params,
            activatable: true,
        });

        const box = new Gtk.Box({
            valign: Gtk.Align.CENTER,
        });
        box.add(new Gtk.Image({
            icon_name: 'pan-down-symbolic',
            visible: true,
        }));
        this.add_suffix(box);

        this._popover = new Gtk.Popover({
            relative_to: box,
        });

        this._listBox = new Gtk.ListBox({
            activate_on_single_click: true,
            selection_mode: Gtk.SelectionMode.SINGLE,
            visible: true,
        });
        this._popover.add(this._listBox);

        this._listBox.connect('row-activated', (view, row) => {
            this._selectedItem = row;
            this.notify('selected-item');
            this._popover.popdown();
        });
    }

    get selected_item() {
        return this._selectedItem;
    }

    add_row(row) {
        row.show();
        this._listBox.add(row);
    }

    activate() {
        this._popover.popup();
    }
});

const GenericPrefs = GObject.registerClass(
class GenericPrefs extends PreferencesGroup {
    _init() {
        super._init();

        this._actionGroup = new Gio.SimpleActionGroup();
        this.insert_action_group('options', this._actionGroup);

        this._actionGroup.add_action(settings.create_action('top-banner'));
        this._actionGroup.add_action(settings.create_action('bottom-banner'));
        this._actionGroup.add_action(settings.create_action('system-info'));

        this.add(this._createSettingsRow('Top banner', 'top-banner'));
        this.add(this._createSettingsRow('Bottom banner', 'bottom-banner'));
        this.add(this._createSettingsRow('System info', 'system-info'));
    }

    _createSettingsRow(title, key) {
        const activatableWidget = new Gtk.Switch({
            valign: Gtk.Align.CENTER,
            action_name: `options.${key}`,
        });
        const row = new ActionRow({
            activatableWidget,
            title,
        });
        row.add_prefix(activatableWidget);

        return row;
    }
});

const BannerPreset = GObject.registerClass({
    Properties: {
        'message': GObject.ParamSpec.string(
            'message', 'message', 'message',
            GObject.ParamFlags.READWRITE,
            null),
        'color': GObject.ParamSpec.string(
            'color', 'color', 'color',
            GObject.ParamFlags.READWRITE,
            null),
        'background-color': GObject.ParamSpec.string(
            'background-color', 'background-color', 'background-color',
            GObject.ParamFlags.READWRITE,
            null),
    },
}, class BannerPreset extends Gtk.ListBoxRow {
    _init(params) {
        super._init(params);

        const label = new Gtk.Label({
            label: this.message,
            visible: true,
        });
        this.add(label);

        const provider = new Gtk.CssProvider();
        provider.load_from_data(`* {
            border-radius: 99px;
            padding: 6px;
            color: ${this.color};
            background-color: ${this.background_color};
        }`);
        label.get_style_context().add_provider(provider,
            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION);
    }
});

const AppearancePrefs = GObject.registerClass(
class AppearancePrefs extends PreferencesGroup {
    _init() {
        super._init();

        let row, activatableWidget;
        row = new ComboRow({
            title: _('Presets'),
        });
        row.add_row(new BannerPreset({
            message: 'UNCLASSIFIED',
            color: '#fff',
            background_color: 'rgba(0, 122, 51, 0.75)',
        }));
        row.add_row(new BannerPreset({
            message: 'CONFIDENTIAL',
            color: '#fff',
            background_color: 'rgba(0, 51, 160, 0.75)',
        }));
        row.add_row(new BannerPreset({
            message: 'SECRET',
            color: '#fff',
            background_color: 'rgba(200, 16, 46, 0.75)',
        }));
        row.add_row(new BannerPreset({
            message: 'TOP SECRET',
            color: '#fff',
            background_color: 'rgba(255, 103, 31, 0.75)',
        }));
        row.add_row(new BannerPreset({
            message: 'TOP SECRET//SCI',
            color: '#000',
            background_color: 'rgba(247, 234, 72, 0.75)',
        }));

        row.connect('notify::selected-item', comboRow => {
            const { message, color, background_color: backgroundColor } =
                comboRow.selected_item;
            settings.set_string('message', message);
            settings.set_string('color', color);
            settings.set_string('background-color', backgroundColor);
        });
        this.add(row);

        activatableWidget = new Gtk.Entry({
            valign: Gtk.Align.CENTER,
        });
        settings.bind('message',
            activatableWidget, 'text',
            Gio.SettingsBindFlags.DEFAULT);
        row = new ActionRow({ title: _('Message'), activatableWidget });
        row.add_suffix(activatableWidget);
        this.add(row);

        activatableWidget = this._createColorButton('background-color', {
            use_alpha: true,
        });
        row = new ActionRow({ title: _('Background color'), activatableWidget });
        row.add_suffix(activatableWidget);
        this.add(row);

        activatableWidget = this._createColorButton('color');
        row = new ActionRow({ title: _('Text color'), activatableWidget });
        row.add_suffix(activatableWidget);
        this.add(row);
    }

    _createColorButton(key, params = {}) {
        const rgba = new Gdk.RGBA();
        rgba.parse(settings.get_string(key));

        const button = new Gtk.ColorButton({
            ...params,
            rgba,
            valign: Gtk.Align.CENTER,
        });
        settings.connect(`changed::${key}`, () => {
            const newRgba = new Gdk.RGBA();
            newRgba.parse(settings.get_string(key));
            if (!newRgba.equal(button.rgba))
                button.set({ rgba: newRgba });
        });
        button.connect('notify::rgba',
            () => settings.set_string(key, button.rgba.to_string()));
        return button;
    }
});

const ClassificationPrefs = GObject.registerClass(
class ClassificationPrefs extends Gtk.ScrolledWindow {
    _init() {
        super._init({
            hscrollbar_policy: Gtk.PolicyType.NEVER,
        });

        const box = new Gtk.Box({
            orientation: Gtk.Orientation.VERTICAL,
            halign: Gtk.Align.CENTER,
            visible: true,
            spacing: 24,
            margin_top: 24,
            margin_bottom: 24,
            margin_start: 12,
            margin_end: 12,
        });
        this.add(box);

        const provider = new Gtk.CssProvider();
        provider.load_from_data('* { min-width: 500px; }');
        box.get_style_context().add_provider(provider,
            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION);

        box.add(new AppearancePrefs());
        box.add(new GenericPrefs());

        this.show_all();
    }
});

/** */
function init() {
    ExtensionUtils.initTranslations();
    settings = ExtensionUtils.getSettings();
}

/**
 * @returns {Gtk.Widget} - the prefs widget
 */
function buildPrefsWidget() {
    return new ClassificationPrefs();
}
